/*
 * TINITalk.java - Toggle serial DTR and send commands (e.g., clear TINI's
 *                 heap, restart slush, set IP address, etc.).
 *
 * Version 1.1  Oct 8 1999
 * Brent J. Nordquist <bjn@visi.com>
 *
 *
 * Code based on:
 *
 * @(#)SimpleRead.java  1.12 98/06/25 SMI
 * @(#)SimpleWrite.java 1.12 98/06/25 SMI
 *
 * Copyright (c) 1998 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license 
 * to use, modify and redistribute this software in source and binary
 * code form, provided that i) this copyright notice and license appear
 * on all copies of the software; and ii) Licensee does not utilize the
 * software in a manner which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind.
 * ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
 * INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND
 * ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
 * LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
 * SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS LICENSORS
 * BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING
 * OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control
 * of aircraft, air traffic, aircraft navigation or aircraft
 * communications; or in the design, construction, operation or
 * maintenance of any nuclear facility. Licensee represents and
 * warrants that it will not use or redistribute the Software for such
 * purposes.
 */

import java.io.*;
import java.util.*;
import javax.comm.*;

public class TINITalk implements Runnable, SerialPortEventListener {
    static CommPortIdentifier portId;
    static Enumeration portList;
//    static String portName = "COM1";  // DOS
//    static String portName = "/dev/term/a";  // Solaris
    static String portName = "/dev/ttyS0";  // Linux

    InputStream inputStream;
    OutputStream outputStream;
    BufferedReader fileBuffer;
    SerialPort serialPort;
    Thread readThread;
    String line;
    int expectChar;

    public static void main(String[] args) {
        portList = CommPortIdentifier.getPortIdentifiers();

        while (portList.hasMoreElements()) {
            portId = (CommPortIdentifier) portList.nextElement();
            if (portId.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                if (portId.getName().equals(portName)) {
                    TINITalk talker = new TINITalk(args);
                }
            }
        }
    }

    public TINITalk(String[] args) {
        String fileName;

        // name of command file (from command line, otherwise default)
        fileName = (args.length > 0) ? args[0] : "clearheap.txt";
        try {
            serialPort = (SerialPort) portId.open("TINITalkApp", 2000);
        } catch (PortInUseException e) {}
        try {
            inputStream = serialPort.getInputStream();
            outputStream = serialPort.getOutputStream();
            fileBuffer = new BufferedReader(new FileReader(fileName));
            expectChar = 0;
        } catch (FileNotFoundException e) {
            System.err.println("ERROR: can't open command file: " + fileName);
            return;
        } catch (IOException e) {}
        try {
            serialPort.addEventListener(this);
        } catch (TooManyListenersException e) {}
        serialPort.notifyOnDataAvailable(true);
        try {
            serialPort.setSerialPortParams(115200,
                SerialPort.DATABITS_8,
                SerialPort.STOPBITS_1,
                SerialPort.PARITY_NONE);
            serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);

            // raise DTR for .2 seconds to reset the TINI board
            serialPort.setDTR(true);
            System.err.print("{raised DTR}");
            System.err.flush();
            try {
                Thread.sleep(200);
            } catch (InterruptedException e) {}
            serialPort.setDTR(false);
            System.err.print("{lowered DTR}");
            System.err.flush();
        } catch (UnsupportedCommOperationException e) {}

        // send CR within first few seconds after reset, to get prompt
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {}
        try {
            outputStream.write('\r');
        } catch (IOException e) {}
        System.err.print("{sent CR}");
        System.err.flush();

        // launch serial reader thread
        readThread = new Thread(this);
        readThread.start();
    }

    public void run() {
        try {
            Thread.sleep(20000);
        } catch (InterruptedException e) {}
    }

    public void serialEvent(SerialPortEvent event) {
        switch(event.getEventType()) {
        case SerialPortEvent.BI:
        case SerialPortEvent.OE:
        case SerialPortEvent.FE:
        case SerialPortEvent.PE:
        case SerialPortEvent.CD:
        case SerialPortEvent.CTS:
        case SerialPortEvent.DSR:
        case SerialPortEvent.RI:
        case SerialPortEvent.OUTPUT_BUFFER_EMPTY:
            break;
        case SerialPortEvent.DATA_AVAILABLE:
            try {
                while (inputStream.available() > 0) {
                    sendExpect();
                }
            } catch (IOException e) {}
            break;
        }
    }

    public void sendExpect() {
        int ch;

        // first character of command file is character to expect
        if (expectChar == 0) {
            try {
                expectChar = fileBuffer.read();
            } catch (IOException e) {}
            System.err.print("{expecting '");
            System.err.write(expectChar);
            System.err.print("'}");
            System.err.flush();
        }

        try {
            // read a character from serial, print it, match against expected
            ch = inputStream.read();
            System.out.write(ch);
            System.out.flush();
            if (ch == expectChar) {
                System.err.print("{saw '");
                System.err.write(expectChar);
                System.err.print("'}");
                System.err.flush();
                // next time around, read another expected character:
                expectChar = 0;

                // read line from command file to write to serial
                line = fileBuffer.readLine();
                if (line.length() == 0) {
                    // deal with empty line case first
                    outputStream.write('\r');
                    System.err.print("{sent CR}");
                } else if (line.charAt(line.length() - 1) == '\\') {
                    // trailing backslash means send text *without* CR
                    line = line.substring(0, line.length() - 1);
                    if (line.length() > 0) {
                        outputStream.write(line.getBytes());
                        System.err.print("{sent '" + line + "'}");
                    } else {
                        System.err.print("{sent nothing}");
                    }
                } else {
                    // normal line:  send text followed by CR
                    outputStream.write(line.getBytes());
                    outputStream.write('\r');
                    System.err.print("{sent '" + line + "'+CR}");
                }
                System.err.flush();

                // end of command file?
                if (!fileBuffer.ready()) {
                   System.err.println("{end of commands}");
                   (Runtime.getRuntime()).exit(0);
                }
            }
        } catch (IOException e) {}
    }
}
